@@ -48,7 +48,7 @@ use std::sync::{Arc, Mutex};
 
 use arti_client::{IntoTorAddr, TorClient};
 use ureq::{
-    http::{uri::Scheme, Uri},
+    http::{Uri, uri::Scheme},
     tls::TlsProvider as UreqTlsProvider,
     unversioned::{
         resolver::{ArrayVec, ResolvedSocketAddrs, Resolver as UreqResolver},
@@ -58,7 +58,7 @@ use ureq::{
 
 use educe::Educe;
 use thiserror::Error;
-use tor_proto::stream::{DataReader, DataWriter};
+use tor_proto::client::stream::{DataReader, DataWriter};
 use tor_rtcompat::{Runtime, ToplevelBlockOn};
 
 #[cfg(feature = "rustls")]
@@ -165,7 +165,7 @@ pub struct Connector<R: Runtime> {
     #[educe(Debug(ignore))]
     client: TorClient<R>,
 
-    /// Selected [`ureq::tls::TlsProvider`]. Possibile options are `Rustls` or `NativeTls`. The default is `Rustls`.
+    /// Selected [`ureq::tls::TlsProvider`]. Possible options are `Rustls` or `NativeTls`. The default is `Rustls`.
     tls_provider: UreqTlsProvider,
 }
 
@@ -374,16 +374,18 @@ impl<R: Runtime + ToplevelBlockOn> Transport for HttpTransport<R> {
     }
 }
 
-impl<R: Runtime> ConnectorBuilder<R> {
+impl ConnectorBuilder<tor_rtcompat::PreferredRuntime> {
     /// Returns instance of [`ConnectorBuilder`] with default values.
-    pub fn new() -> Result<ConnectorBuilder<tor_rtcompat::PreferredRuntime>, Error> {
+    pub fn new() -> Result<Self, Error> {
         Ok(ConnectorBuilder {
             client: None,
             runtime: tor_rtcompat::PreferredRuntime::create()?,
             tls_provider: None,
         })
     }
+}
 
+impl<R: Runtime> ConnectorBuilder<R> {
     /// Creates instance of [`Connector`] from the builder.
     pub fn build(self) -> Result<Connector<R>, Error> {
         let client = match self.client {
@@ -616,7 +618,7 @@ pub fn get_default_tls_provider() -> UreqTlsProvider {
 /// ```rust,no_run
 /// let rt = tor_rtcompat::PreferredRuntime::create().expect("Failed to create runtime.");
 /// let tls_provider = arti_ureq::get_default_tls_provider();
-///    
+///
 /// let client = arti_client::TorClient::with_runtime(rt.clone())
 ///     .create_unbootstrapped()
 ///     .expect("Error creating Tor Client.");
@@ -628,10 +630,10 @@ pub fn get_default_tls_provider() -> UreqTlsProvider {
 ///
 /// let arti_connector = builder.build();
 /// ```
-impl<R: Runtime> Connector<R> {
+impl Connector<tor_rtcompat::PreferredRuntime> {
     /// Returns new [`ConnectorBuilder`] with default values.
     pub fn builder() -> Result<ConnectorBuilder<tor_rtcompat::PreferredRuntime>, Error> {
-        ConnectorBuilder::<R>::new()
+        ConnectorBuilder::new()
     }
 }
 
@@ -681,7 +683,7 @@ mod arti_ureq_test {
     // complex to use the `==` operator or (Partial)Eq. So we compare the types of the individual properties instead.
     fn assert_equal_types<T>(_: &T, _: &T) {}
 
-    // Helper function to check if the enviroment variable ARTI_TEST_LIVE_NETWORK is set to 1.
+    // Helper function to check if the environment variable ARTI_TEST_LIVE_NETWORK is set to 1.
     // We only want to run tests using the live network when the user explicitly wants to.
     fn test_live_network() -> bool {
         let run_test = std::env::var(ARTI_TEST_LIVE_NETWORK).is_ok_and(|v| v == "1");
@@ -692,7 +694,7 @@ mod arti_ureq_test {
         run_test
     }
 
-    // Helper function to check if the enviroment variable ARTI_TESTING_ON_LOCAL is set to 1.
+    // Helper function to check if the environment variable ARTI_TESTING_ON_LOCAL is set to 1.
     // Some tests, especially those that create default `Connector` instances, or test the `ConnectorBuilder`,
     // are not reliable when run on CI.  We don't know why that is.  It's probably a bug.  TODO fix the tests!
     fn testing_on_local() -> bool {
@@ -740,13 +742,11 @@ mod arti_ureq_test {
             .expect("Failed to read body.");
         let json_response: serde_json::Value =
             serde_json::from_str(&response).expect("Failed to parse JSON.");
-        let is_tor = json_response
+        json_response
             .get("IsTor")
             .expect("Failed to retrieve IsTor property from response")
             .as_bool()
-            .expect("Failed to convert IsTor to bool");
-
-        is_tor
+            .expect("Failed to convert IsTor to bool")
     }
 
     // Quick internal test to check if our helper function `equal_types` works as expected.
