@@ -5,14 +5,16 @@ use std::sync::{Arc, Mutex};
 
 use crate::factory::{BootstrapReporter, ChannelFactory, IncomingChannelFactory};
 use crate::transport::TransportImplHelper;
-use crate::{event::ChanMgrEventSender, Error};
+use crate::{Error, event::ChanMgrEventSender};
 
 use std::time::Duration;
 use tor_error::internal;
+use tor_keymgr::KeyMgr;
 use tor_linkspec::{BridgeAddr, HasChanMethod, IntoOwnedChanTarget, OwnedChanTarget};
+use tor_proto::channel::kist::KistParams;
 use tor_proto::channel::params::ChannelPaddingInstructionsUpdates;
 use tor_proto::memquota::ChannelAccount;
-use tor_rtcompat::{tls::TlsConnector, Runtime, TlsProvider};
+use tor_rtcompat::{Runtime, TlsProvider, tls::TlsConnector};
 
 use async_trait::async_trait;
 use futures::task::SpawnExt;
@@ -39,6 +41,9 @@ where
     transport: H,
     /// Object to build TLS connections.
     tls_connector: <R as TlsProvider<H::Stream>>::Connector,
+    /// Relay only: Key manager to get keys for channel authentication.
+    #[expect(dead_code)]
+    keymgr: Option<Arc<KeyMgr>>,
 }
 
 impl<R: Runtime, H: TransportImplHelper> ChanBuilder<R, H>
@@ -46,12 +51,13 @@ where
     R: TlsProvider<H::Stream>,
 {
     /// Construct a new ChanBuilder.
-    pub fn new(runtime: R, transport: H) -> Self {
+    pub fn new(runtime: R, transport: H, keymgr: Option<Arc<KeyMgr>>) -> Self {
         let tls_connector = <R as TlsProvider<H::Stream>>::tls_connector(&runtime);
         ChanBuilder {
             runtime,
             transport,
             tls_connector,
+            keymgr,
         }
     }
 }
@@ -191,7 +197,7 @@ where
         let mut builder = ChannelBuilder::new();
         builder.set_declared_method(using_method);
         let chan = builder
-            .launch(
+            .launch_client(
                 tls,
                 self.runtime.clone(), /* TODO provide ZST SleepProvider instead */
                 memquota,
@@ -253,6 +259,9 @@ impl crate::mgr::AbstractChannel for tor_proto::channel::Channel {
     ) -> tor_proto::Result<()> {
         tor_proto::channel::Channel::reparameterize(self, updates)
     }
+    fn reparameterize_kist(&self, kist_params: KistParams) -> tor_proto::Result<()> {
+        tor_proto::channel::Channel::reparameterize_kist(self, kist_params)
+    }
     fn engage_padding_activities(&self) {
         tor_proto::channel::Channel::engage_padding_activities(self);
     }
@@ -275,8 +284,8 @@ mod test {
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
     use crate::{
-        mgr::{AbstractChannel, AbstractChannelFactory},
         Result,
+        mgr::{AbstractChannel, AbstractChannelFactory},
     };
     use futures::StreamExt as _;
     use std::net::SocketAddr;
@@ -286,8 +295,11 @@ mod test {
     use tor_llcrypto::pk::rsa::RsaIdentity;
     use tor_proto::channel::Channel;
     use tor_proto::memquota::{ChannelAccount, SpecificAccount as _};
-    use tor_rtcompat::{test_with_one_runtime, NetStreamListener};
-    use tor_rtmock::{io::LocalStream, net::MockNetwork, MockSleepRuntime};
+    use tor_rtcompat::{NetStreamListener, test_with_one_runtime};
+    use tor_rtmock::{io::LocalStream, net::MockNetwork};
+
+    #[allow(deprecated)] // TODO #1885
+    use tor_rtmock::MockSleepRuntime;
 
     // Make sure that the builder can build a real channel.  To test
     // this out, we set up a listener that pretends to have the right
@@ -320,6 +332,7 @@ mod test {
                 .add_address(client_addr)
                 .runtime(rt.clone());
             // Mock the current time too
+            #[allow(deprecated)] // TODO #1885
             let client_rt = MockSleepRuntime::new(client_rt);
 
             // Set up a relay runtime with a different IP
@@ -336,7 +349,7 @@ mod test {
 
             // Create the channel builder that we want to test.
             let transport = crate::transport::DefaultTransport::new(client_rt.clone());
-            let builder = ChanBuilder::new(client_rt, transport);
+            let builder = ChanBuilder::new(client_rt, transport, None);
 
             let (r1, r2): (Result<Arc<Channel>>, Result<LocalStream>) = futures::join!(
                 async {
