@@ -10,8 +10,8 @@
 //! authorities' votes, and signed by multiple authorities.
 //!
 //! A consensus document can itself come in two different flavors: a
-//! "ns"-flavored consensus has references to router descriptors, and
-//! a "microdesc"-flavored consensus has references to
+//! plain (unflavoured) consensus has references to router descriptors, and
+//! a "microdesc"-flavored consensus ("md") has references to
 //! microdescriptors.
 //!
 //! To keep an up-to-date view of the network, clients download
@@ -35,7 +35,7 @@
 //!
 //! TODO: This module doesn't implement vote parsing at all yet.
 //!
-//! TODO: This module doesn't implement ns-flavored consensuses.
+//! TODO: This module doesn't implement plain consensuses.
 //!
 //! TODO: More testing is needed!
 //!
@@ -45,6 +45,12 @@
 
 mod rs;
 
+pub mod md;
+#[cfg(feature = "plain-consensus")]
+pub mod plain;
+#[cfg(feature = "ns-vote")]
+pub mod vote;
+
 #[cfg(feature = "build_docs")]
 mod build;
 
@@ -53,35 +59,56 @@ use crate::parse::keyword::Keyword;
 use crate::parse::parser::{Section, SectionRules, SectionRulesBuilder};
 use crate::parse::tokenize::{Item, ItemResult, NetDocReader};
 use crate::types::misc::*;
-use crate::util::private::Sealed;
 use crate::util::PeekableIterator;
 use crate::{Error, NetdocErrorKind as EK, Pos, Result};
 use std::collections::{HashMap, HashSet};
 use std::result::Result as StdResult;
 use std::sync::Arc;
 use std::{net, result, time};
-use tor_error::{internal, HasKind};
+use tor_error::{HasKind, internal};
 use tor_protover::Protocols;
 
 use bitflags::bitflags;
 use digest::Digest;
-use once_cell::sync::Lazy;
-use tor_checkable::{timed::TimerangeBound, ExternallySigned};
+use std::sync::LazyLock;
+use tor_checkable::{ExternallySigned, timed::TimerangeBound};
 use tor_llcrypto as ll;
 use tor_llcrypto::pk::rsa::RsaIdentity;
 
 use serde::{Deserialize, Deserializer};
 
 #[cfg(feature = "build_docs")]
-pub use build::ConsensusBuilder;
+pub use build::MdConsensusBuilder;
+#[cfg(all(feature = "build_docs", feature = "plain-consensus"))]
+pub use build::PlainConsensusBuilder;
 #[cfg(feature = "build_docs")]
-pub use rs::build::RouterStatusBuilder;
+ns_export_each_flavor! {
+    ty: RouterStatusBuilder;
+}
+
+ns_export_each_variety! {
+    ty: RouterStatus;
+}
+#[cfg(feature = "dangerous-expose-struct-fields")]
+ns_export_each_variety! {
+    ty: Header;
+}
 
-pub use rs::MdConsensusRouterStatus;
-#[cfg(feature = "ns_consensus")]
-pub use rs::NsConsensusRouterStatus;
 use void::ResultVoidExt as _;
 
+#[deprecated]
+#[cfg(feature = "ns_consensus")]
+pub use PlainConsensus as NsConsensus;
+#[deprecated]
+#[cfg(feature = "ns_consensus")]
+pub use PlainRouterStatus as NsRouterStatus;
+#[deprecated]
+#[cfg(feature = "ns_consensus")]
+pub use UncheckedPlainConsensus as UncheckedNsConsensus;
+#[deprecated]
+#[cfg(feature = "ns_consensus")]
+pub use UnvalidatedPlainConsensus as UnvalidatedNsConsensus;
+
 /// The lifetime of a networkstatus document.
 ///
 /// In a consensus, this type describes when the consensus may safely
@@ -326,25 +353,25 @@ pub enum ConsensusFlavor {
     /// historical and network-health purposes.  Instead of listing
     /// microdescriptor digests, it lists digests of full relay
     /// descriptors.
-    Ns,
+    Plain,
 }
 
 impl ConsensusFlavor {
     /// Return the name of this consensus flavor.
     pub fn name(&self) -> &'static str {
         match self {
-            ConsensusFlavor::Ns => "ns",
+            ConsensusFlavor::Plain => "ns", // spec bug, now baked in
             ConsensusFlavor::Microdesc => "microdesc",
         }
     }
     /// Try to find the flavor whose name is `name`.
     ///
-    /// For historical reasons, an unnamed flavor indicates an "Ns"
+    /// For historical reasons, an unnamed flavor indicates an "Plain"
     /// document.
     pub fn from_opt_name(name: Option<&str>) -> Result<Self> {
         match name {
             Some("microdesc") => Ok(ConsensusFlavor::Microdesc),
-            Some("ns") | None => Ok(ConsensusFlavor::Ns),
+            Some("ns") | None => Ok(ConsensusFlavor::Plain),
             Some(other) => {
                 Err(EK::BadDocumentType.with_msg(format!("unrecognized flavor {:?}", other)))
             }
@@ -433,73 +460,6 @@ pub struct SharedRandStatus {
     timestamp: Option<time::SystemTime>,
 }
 
-/// Parts of the networkstatus header that are present in every networkstatus.
-///
-/// NOTE: this type is separate from the header parts that are only in
-/// votes or only in consensuses, even though we don't implement votes yet.
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    visibility::make(pub),
-    non_exhaustive
-)]
-#[derive(Debug, Clone)]
-struct CommonHeader {
-    /// What kind of consensus document is this?  Absent in votes and
-    /// in ns-flavored consensuses.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    flavor: ConsensusFlavor,
-    /// Over what time is this consensus valid?  (For votes, this is
-    /// the time over which the voted-upon consensus should be valid.)
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    lifetime: Lifetime,
-    /// List of recommended Tor client versions.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    client_versions: Vec<String>,
-    /// List of recommended Tor relay versions.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    relay_versions: Vec<String>,
-    /// Lists of recommended and required subprotocols.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    proto_statuses: Arc<ProtoStatuses>,
-    /// Declared parameters for tunable settings about how to the
-    /// network should operator. Some of these adjust timeouts and
-    /// whatnot; some features things on and off.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    params: NetParams<i32>,
-    /// How long in seconds should voters wait for votes and
-    /// signatures (respectively) to propagate?
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    voting_delay: Option<(u32, u32)>,
-}
-
-/// The header of a consensus networkstatus.
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    visibility::make(pub),
-    non_exhaustive
-)]
-#[derive(Debug, Clone)]
-struct ConsensusHeader {
-    /// Header fields common to votes and consensuses
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    hdr: CommonHeader,
-    /// What "method" was used to produce this consensus?  (A
-    /// consensus method is a version number used by authorities to
-    /// upgrade the consensus algorithm.)
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    consensus_method: u32,
-    /// Global shared-random value for the previous shared-random period.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    shared_rand_prev: Option<SharedRandStatus>,
-    /// Global shared-random value for the current shared-random period.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    shared_rand_cur: Option<SharedRandStatus>,
-}
-
 /// Description of an authority's identity and address.
 ///
 /// (Corresponds to a dir-source line.)
@@ -626,7 +586,7 @@ impl RelayWeight {
     non_exhaustive
 )]
 #[derive(Debug, Clone)]
-struct ConsensusVoterInfo {
+pub(crate) struct ConsensusVoterInfo {
     /// Contents of the dirsource line about an authority
     #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
     dir_source: DirSource,
@@ -648,7 +608,7 @@ struct ConsensusVoterInfo {
     non_exhaustive
 )]
 #[derive(Debug, Clone)]
-struct Footer {
+pub(crate) struct Footer {
     /// Weights to be applied to certain classes of relays when choosing
     /// for different roles.
     ///
@@ -658,143 +618,32 @@ struct Footer {
     weights: NetParams<i32>,
 }
 
-/// Trait to parse a single relay as listed in a consensus document.
-///
-/// TODO(nickm): I'd rather not have this trait be public, but I haven't yet
-/// figured out how to make it private.
-pub trait ParseRouterStatus: Sized + Sealed {
-    /// Parse this object from a `Section` object containing its
-    /// elements.
-    fn from_section(sec: &Section<'_, NetstatusKwd>) -> Result<Self>;
-
-    /// Return the networkstatus consensus flavor in which this
-    /// routerstatus appears.
-    fn flavor() -> ConsensusFlavor;
-}
-
-/// Represents a single relay as listed in a consensus document.
-///
-/// Not implementable outside of the `tor-netdoc` crate.
-pub trait RouterStatus: Sealed {
-    /// A digest of the document that's identified by this RouterStatus.
-    type DocumentDigest: Clone;
-
-    /// Return RSA identity for the relay described by this RouterStatus
-    fn rsa_identity(&self) -> &RsaIdentity;
-
-    /// Return the digest of the document identified by this
-    /// routerstatus.
-    fn doc_digest(&self) -> &Self::DocumentDigest;
-}
-
-/// A single microdescriptor consensus netstatus
-///
-/// TODO: This should possibly turn into a parameterized type, to represent
-/// votes and ns consensuses.
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
-#[derive(Debug, Clone)]
-pub struct Consensus<RS> {
-    /// Part of the header shared by all consensus types.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    header: ConsensusHeader,
-    /// List of voters whose votes contributed to this consensus.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    voters: Vec<ConsensusVoterInfo>,
-    /// A list of routerstatus entries for the relays on the network,
-    /// with one entry per relay.
-    ///
-    /// These are currently ordered by the router's RSA identity, but this is not
-    /// to be relied on, since we may want to even abolish RSA at some point!
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    relays: Vec<RS>,
-    /// Footer for the consensus object.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    footer: Footer,
-}
-
 /// A consensus document that lists relays along with their
 /// microdescriptor documents.
-pub type MdConsensus = Consensus<MdConsensusRouterStatus>;
+pub type MdConsensus = md::Consensus;
 
 /// An MdConsensus that has been parsed and checked for timeliness,
 /// but not for signatures.
-pub type UnvalidatedMdConsensus = UnvalidatedConsensus<MdConsensusRouterStatus>;
+pub type UnvalidatedMdConsensus = md::UnvalidatedConsensus;
 
 /// An MdConsensus that has been parsed but not checked for signatures
 /// and timeliness.
-pub type UncheckedMdConsensus = UncheckedConsensus<MdConsensusRouterStatus>;
+pub type UncheckedMdConsensus = md::UncheckedConsensus;
 
-#[cfg(feature = "ns_consensus")]
+#[cfg(feature = "plain-consensus")]
 /// A consensus document that lists relays along with their
 /// router descriptor documents.
-pub type NsConsensus = Consensus<NsConsensusRouterStatus>;
+pub type PlainConsensus = plain::Consensus;
 
-#[cfg(feature = "ns_consensus")]
-/// An NsConsensus that has been parsed and checked for timeliness,
+#[cfg(feature = "plain-consensus")]
+/// An PlainConsensus that has been parsed and checked for timeliness,
 /// but not for signatures.
-pub type UnvalidatedNsConsensus = UnvalidatedConsensus<NsConsensusRouterStatus>;
+pub type UnvalidatedPlainConsensus = plain::UnvalidatedConsensus;
 
-#[cfg(feature = "ns_consensus")]
-/// An NsConsensus that has been parsed but not checked for signatures
+#[cfg(feature = "plain-consensus")]
+/// An PlainConsensus that has been parsed but not checked for signatures
 /// and timeliness.
-pub type UncheckedNsConsensus = UncheckedConsensus<NsConsensusRouterStatus>;
-
-impl<RS> Consensus<RS> {
-    /// Return the Lifetime for this consensus.
-    pub fn lifetime(&self) -> &Lifetime {
-        &self.header.hdr.lifetime
-    }
-
-    /// Return a slice of all the routerstatus entries in this consensus.
-    pub fn relays(&self) -> &[RS] {
-        &self.relays[..]
-    }
-
-    /// Return a mapping from keywords to integers representing how
-    /// to weight different kinds of relays in different path positions.
-    pub fn bandwidth_weights(&self) -> &NetParams<i32> {
-        &self.footer.weights
-    }
-
-    /// Return the map of network parameters that this consensus advertises.
-    pub fn params(&self) -> &NetParams<i32> {
-        &self.header.hdr.params
-    }
-
-    /// Return the latest shared random value, if the consensus
-    /// contains one.
-    pub fn shared_rand_cur(&self) -> Option<&SharedRandStatus> {
-        self.header.shared_rand_cur.as_ref()
-    }
-
-    /// Return the previous shared random value, if the consensus
-    /// contains one.
-    pub fn shared_rand_prev(&self) -> Option<&SharedRandStatus> {
-        self.header.shared_rand_prev.as_ref()
-    }
-
-    /// Return a [`ProtoStatus`] that lists the network's current requirements and
-    /// recommendations for the list of protocols that every relay must implement.  
-    pub fn relay_protocol_status(&self) -> &ProtoStatus {
-        &self.header.hdr.proto_statuses.relay
-    }
-
-    /// Return a [`ProtoStatus`] that lists the network's current requirements and
-    /// recommendations for the list of protocols that every client must implement.
-    pub fn client_protocol_status(&self) -> &ProtoStatus {
-        &self.header.hdr.proto_statuses.client
-    }
-
-    /// Return a set of all known [`ProtoStatus`] values.
-    pub fn protocol_statuses(&self) -> &Arc<ProtoStatuses> {
-        &self.header.hdr.proto_statuses
-    }
-}
+pub type UncheckedPlainConsensus = plain::UncheckedConsensus;
 
 decl_keyword! {
     /// Keywords that can be used in votes and consensuses.
@@ -865,7 +714,7 @@ decl_keyword! {
 }
 
 /// Shared parts of rules for all kinds of netstatus headers
-static NS_HEADER_RULES_COMMON_: Lazy<SectionRulesBuilder<NetstatusKwd>> = Lazy::new(|| {
+static NS_HEADER_RULES_COMMON_: LazyLock<SectionRulesBuilder<NetstatusKwd>> = LazyLock::new(|| {
     use NetstatusKwd::*;
     let mut rules = SectionRules::builder();
     rules.add(NETWORK_STATUS_VERSION.rule().required().args(1..=2));
@@ -885,7 +734,7 @@ static NS_HEADER_RULES_COMMON_: Lazy<SectionRulesBuilder<NetstatusKwd>> = Lazy::
     rules
 });
 /// Rules for parsing the header of a consensus.
-static NS_HEADER_RULES_CONSENSUS: Lazy<SectionRules<NetstatusKwd>> = Lazy::new(|| {
+static NS_HEADER_RULES_CONSENSUS: LazyLock<SectionRules<NetstatusKwd>> = LazyLock::new(|| {
     use NetstatusKwd::*;
     let mut rules = NS_HEADER_RULES_COMMON_.clone();
     rules.add(CONSENSUS_METHOD.rule().args(1..=1));
@@ -924,7 +773,7 @@ static NS_VOTERINFO_RULES_VOTE: SectionRules<NetstatusKwd> = {
 };
  */
 /// Rules for parsing a single voter's information in a consensus
-static NS_VOTERINFO_RULES_CONSENSUS: Lazy<SectionRules<NetstatusKwd>> = Lazy::new(|| {
+static NS_VOTERINFO_RULES_CONSENSUS: LazyLock<SectionRules<NetstatusKwd>> = LazyLock::new(|| {
     use NetstatusKwd::*;
     let mut rules = SectionRules::builder();
     rules.add(DIR_SOURCE.rule().required().args(6..));
@@ -934,21 +783,22 @@ static NS_VOTERINFO_RULES_CONSENSUS: Lazy<SectionRules<NetstatusKwd>> = Lazy::ne
     rules.build()
 });
 /// Shared rules for parsing a single routerstatus
-static NS_ROUTERSTATUS_RULES_COMMON_: Lazy<SectionRulesBuilder<NetstatusKwd>> = Lazy::new(|| {
-    use NetstatusKwd::*;
-    let mut rules = SectionRules::builder();
-    rules.add(RS_A.rule().may_repeat().args(1..));
-    rules.add(RS_S.rule().required());
-    rules.add(RS_V.rule());
-    rules.add(RS_PR.rule().required());
-    rules.add(RS_W.rule());
-    rules.add(RS_P.rule().args(2..));
-    rules.add(UNRECOGNIZED.rule().may_repeat().obj_optional());
-    rules
-});
+static NS_ROUTERSTATUS_RULES_COMMON_: LazyLock<SectionRulesBuilder<NetstatusKwd>> =
+    LazyLock::new(|| {
+        use NetstatusKwd::*;
+        let mut rules = SectionRules::builder();
+        rules.add(RS_A.rule().may_repeat().args(1..));
+        rules.add(RS_S.rule().required());
+        rules.add(RS_V.rule());
+        rules.add(RS_PR.rule().required());
+        rules.add(RS_W.rule());
+        rules.add(RS_P.rule().args(2..));
+        rules.add(UNRECOGNIZED.rule().may_repeat().obj_optional());
+        rules
+    });
 
 /// Rules for parsing a single routerstatus in an NS consensus
-static NS_ROUTERSTATUS_RULES_NSCON: Lazy<SectionRules<NetstatusKwd>> = Lazy::new(|| {
+static NS_ROUTERSTATUS_RULES_PLAIN: LazyLock<SectionRules<NetstatusKwd>> = LazyLock::new(|| {
     use NetstatusKwd::*;
     let mut rules = NS_ROUTERSTATUS_RULES_COMMON_.clone();
     rules.add(RS_R.rule().required().args(8..));
@@ -967,7 +817,7 @@ static NS_ROUTERSTATUS_RULES_VOTE: SectionRules<NetstatusKwd> = {
     };
 */
 /// Rules for parsing a single routerstatus in a microdesc consensus
-static NS_ROUTERSTATUS_RULES_MDCON: Lazy<SectionRules<NetstatusKwd>> = Lazy::new(|| {
+static NS_ROUTERSTATUS_RULES_MDCON: LazyLock<SectionRules<NetstatusKwd>> = LazyLock::new(|| {
     use NetstatusKwd::*;
     let mut rules = NS_ROUTERSTATUS_RULES_COMMON_.clone();
     rules.add(RS_R.rule().required().args(6..));
@@ -975,7 +825,7 @@ static NS_ROUTERSTATUS_RULES_MDCON: Lazy<SectionRules<NetstatusKwd>> = Lazy::new
     rules.build()
 });
 /// Rules for parsing consensus fields from a footer.
-static NS_FOOTER_RULES: Lazy<SectionRules<NetstatusKwd>> = Lazy::new(|| {
+static NS_FOOTER_RULES: LazyLock<SectionRules<NetstatusKwd>> = LazyLock::new(|| {
     use NetstatusKwd::*;
     let mut rules = SectionRules::builder();
     rules.add(DIRECTORY_FOOTER.rule().required().no_args());
@@ -1066,99 +916,6 @@ where
     }
 }
 
-impl CommonHeader {
-    /// Extract the CommonHeader members from a single header section.
-    fn from_section(sec: &Section<'_, NetstatusKwd>) -> Result<CommonHeader> {
-        use NetstatusKwd::*;
-
-        {
-            // this unwrap is safe because if there is not at least one
-            // token in the section, the section is unparsable.
-            #[allow(clippy::unwrap_used)]
-            let first = sec.first_item().unwrap();
-            if first.kwd() != NETWORK_STATUS_VERSION {
-                return Err(EK::UnexpectedToken
-                    .with_msg(first.kwd().to_str())
-                    .at_pos(first.pos()));
-            }
-        }
-
-        let ver_item = sec.required(NETWORK_STATUS_VERSION)?;
-
-        let version: u32 = ver_item.parse_arg(0)?;
-        if version != 3 {
-            return Err(EK::BadDocumentVersion.with_msg(version.to_string()));
-        }
-        let flavor = ConsensusFlavor::from_opt_name(ver_item.arg(1))?;
-
-        let valid_after = sec
-            .required(VALID_AFTER)?
-            .args_as_str()
-            .parse::<Iso8601TimeSp>()?
-            .into();
-        let fresh_until = sec
-            .required(FRESH_UNTIL)?
-            .args_as_str()
-            .parse::<Iso8601TimeSp>()?
-            .into();
-        let valid_until = sec
-            .required(VALID_UNTIL)?
-            .args_as_str()
-            .parse::<Iso8601TimeSp>()?
-            .into();
-        let lifetime = Lifetime::new(valid_after, fresh_until, valid_until)?;
-
-        let client_versions = sec
-            .maybe(CLIENT_VERSIONS)
-            .args_as_str()
-            .unwrap_or("")
-            .split(',')
-            .map(str::to_string)
-            .collect();
-        let relay_versions = sec
-            .maybe(SERVER_VERSIONS)
-            .args_as_str()
-            .unwrap_or("")
-            .split(',')
-            .map(str::to_string)
-            .collect();
-
-        let proto_statuses = {
-            let client = ProtoStatus::from_section(
-                sec,
-                RECOMMENDED_CLIENT_PROTOCOLS,
-                REQUIRED_CLIENT_PROTOCOLS,
-            )?;
-            let relay = ProtoStatus::from_section(
-                sec,
-                RECOMMENDED_RELAY_PROTOCOLS,
-                REQUIRED_RELAY_PROTOCOLS,
-            )?;
-            Arc::new(ProtoStatuses { client, relay })
-        };
-
-        let params = sec.maybe(PARAMS).args_as_str().unwrap_or("").parse()?;
-
-        let voting_delay = if let Some(tok) = sec.get(VOTING_DELAY) {
-            let n1 = tok.parse_arg(0)?;
-            let n2 = tok.parse_arg(1)?;
-            Some((n1, n2))
-        } else {
-            None
-        };
-
-        Ok(CommonHeader {
-            flavor,
-            lifetime,
-            client_versions,
-            relay_versions,
-            proto_statuses,
-            params,
-            voting_delay,
-        })
-    }
-}
-
 impl SharedRandStatus {
     /// Parse a current or previous shared rand value from a given
     /// SharedRandPreviousValue or SharedRandCurrentValue.
@@ -1170,7 +927,7 @@ impl SharedRandStatus {
                     "wrong keyword {:?} on shared-random value",
                     item.kwd()
                 ))
-                .at_pos(item.pos()))
+                .at_pos(item.pos()));
             }
         }
         let n_reveals: u8 = item.parse_arg(0)?;
@@ -1198,41 +955,6 @@ impl SharedRandStatus {
     }
 }
 
-impl ConsensusHeader {
-    /// Parse the ConsensusHeader members from a provided section.
-    fn from_section(sec: &Section<'_, NetstatusKwd>) -> Result<ConsensusHeader> {
-        use NetstatusKwd::*;
-
-        let status: &str = sec.required(VOTE_STATUS)?.arg(0).unwrap_or("");
-        if status != "consensus" {
-            return Err(EK::BadDocumentType.err());
-        }
-
-        // We're ignoring KNOWN_FLAGS in the consensus.
-
-        let hdr = CommonHeader::from_section(sec)?;
-
-        let consensus_method: u32 = sec.required(CONSENSUS_METHOD)?.parse_arg(0)?;
-
-        let shared_rand_prev = sec
-            .get(SHARED_RAND_PREVIOUS_VALUE)
-            .map(SharedRandStatus::from_item)
-            .transpose()?;
-
-        let shared_rand_cur = sec
-            .get(SHARED_RAND_CURRENT_VALUE)
-            .map(SharedRandStatus::from_item)
-            .transpose()?;
-
-        Ok(ConsensusHeader {
-            hdr,
-            consensus_method,
-            shared_rand_prev,
-            shared_rand_cur,
-        })
-    }
-}
-
 impl DirSource {
     /// Parse a "dir-source" item
     fn from_item(item: &Item<'_, NetstatusKwd>) -> Result<Self> {
@@ -1470,338 +1192,6 @@ impl Signature {
     }
 }
 
-/// A Consensus object that has been parsed, but not checked for
-/// signatures and timeliness.
-pub type UncheckedConsensus<RS> = TimerangeBound<UnvalidatedConsensus<RS>>;
-
-impl<RS: RouterStatus + ParseRouterStatus> Consensus<RS> {
-    /// Return a new ConsensusBuilder for building test consensus objects.
-    ///
-    /// This function is only available when the `build_docs` feature has
-    /// been enabled.
-    #[cfg(feature = "build_docs")]
-    pub fn builder() -> ConsensusBuilder<RS> {
-        ConsensusBuilder::new(RS::flavor())
-    }
-
-    /// Try to parse a single networkstatus document from a string.
-    pub fn parse(s: &str) -> Result<(&str, &str, UncheckedConsensus<RS>)> {
-        let mut reader = NetDocReader::new(s)?;
-        Self::parse_from_reader(&mut reader).map_err(|e| e.within(s))
-    }
-    /// Extract a voter-info section from the reader; return
-    /// Ok(None) when we are out of voter-info sections.
-    fn take_voterinfo(
-        r: &mut NetDocReader<'_, NetstatusKwd>,
-    ) -> Result<Option<ConsensusVoterInfo>> {
-        use NetstatusKwd::*;
-
-        match r.peek() {
-            None => return Ok(None),
-            Some(e) if e.is_ok_with_kwd_in(&[RS_R, DIRECTORY_FOOTER]) => return Ok(None),
-            _ => (),
-        };
-
-        let mut first_dir_source = true;
-        // TODO: Extract this pattern into a "pause at second"???
-        // Pause at the first 'r', or the second 'dir-source'.
-        let mut p = r.pause_at(|i| match i {
-            Err(_) => false,
-            Ok(item) => {
-                item.kwd() == RS_R
-                    || if item.kwd() == DIR_SOURCE {
-                        let was_first = first_dir_source;
-                        first_dir_source = false;
-                        !was_first
-                    } else {
-                        false
-                    }
-            }
-        });
-
-        let voter_sec = NS_VOTERINFO_RULES_CONSENSUS.parse(&mut p)?;
-        let voter = ConsensusVoterInfo::from_section(&voter_sec)?;
-
-        Ok(Some(voter))
-    }
-
-    /// Extract the footer (but not signatures) from the reader.
-    fn take_footer(r: &mut NetDocReader<'_, NetstatusKwd>) -> Result<Footer> {
-        use NetstatusKwd::*;
-        let mut p = r.pause_at(|i| i.is_ok_with_kwd_in(&[DIRECTORY_SIGNATURE]));
-        let footer_sec = NS_FOOTER_RULES.parse(&mut p)?;
-        let footer = Footer::from_section(&footer_sec)?;
-        Ok(footer)
-    }
-
-    /// Extract a routerstatus from the reader.  Return Ok(None) if we're
-    /// out of routerstatus entries.
-    fn take_routerstatus(r: &mut NetDocReader<'_, NetstatusKwd>) -> Result<Option<(Pos, RS)>> {
-        use NetstatusKwd::*;
-        match r.peek() {
-            None => return Ok(None),
-            Some(e) if e.is_ok_with_kwd_in(&[DIRECTORY_FOOTER]) => return Ok(None),
-            _ => (),
-        };
-
-        let pos = r.pos();
-
-        let mut first_r = true;
-        let mut p = r.pause_at(|i| match i {
-            Err(_) => false,
-            Ok(item) => {
-                item.kwd() == DIRECTORY_FOOTER
-                    || if item.kwd() == RS_R {
-                        let was_first = first_r;
-                        first_r = false;
-                        !was_first
-                    } else {
-                        false
-                    }
-            }
-        });
-
-        let rules = match RS::flavor() {
-            ConsensusFlavor::Microdesc => &NS_ROUTERSTATUS_RULES_MDCON,
-            ConsensusFlavor::Ns => &NS_ROUTERSTATUS_RULES_NSCON,
-        };
-
-        let rs_sec = rules.parse(&mut p)?;
-        let rs = RS::from_section(&rs_sec)?;
-        Ok(Some((pos, rs)))
-    }
-
-    /// Extract an entire UncheckedConsensus from a reader.
-    ///
-    /// Returns the signed portion of the string, the remainder of the
-    /// string, and an UncheckedConsensus.
-    fn parse_from_reader<'a>(
-        r: &mut NetDocReader<'a, NetstatusKwd>,
-    ) -> Result<(&'a str, &'a str, UncheckedConsensus<RS>)> {
-        use NetstatusKwd::*;
-        let (header, start_pos) = {
-            let mut h = r.pause_at(|i| i.is_ok_with_kwd_in(&[DIR_SOURCE]));
-            let header_sec = NS_HEADER_RULES_CONSENSUS.parse(&mut h)?;
-            // Unwrapping should be safe because above `.parse` would have
-            // returned an Error
-            #[allow(clippy::unwrap_used)]
-            let pos = header_sec.first_item().unwrap().offset_in(r.str()).unwrap();
-            (ConsensusHeader::from_section(&header_sec)?, pos)
-        };
-        if RS::flavor() != header.hdr.flavor {
-            return Err(EK::BadDocumentType.with_msg(format!(
-                "Expected {:?}, got {:?}",
-                RS::flavor(),
-                header.hdr.flavor
-            )));
-        }
-
-        let mut voters = Vec::new();
-
-        while let Some(voter) = Self::take_voterinfo(r)? {
-            voters.push(voter);
-        }
-
-        let mut relays: Vec<RS> = Vec::new();
-        while let Some((pos, routerstatus)) = Self::take_routerstatus(r)? {
-            if let Some(prev) = relays.last() {
-                if prev.rsa_identity() >= routerstatus.rsa_identity() {
-                    return Err(EK::WrongSortOrder.at_pos(pos));
-                }
-            }
-            relays.push(routerstatus);
-        }
-        relays.shrink_to_fit();
-
-        let footer = Self::take_footer(r)?;
-
-        let consensus = Consensus {
-            header,
-            voters,
-            relays,
-            footer,
-        };
-
-        // Find the signatures.
-        let mut first_sig: Option<Item<'_, NetstatusKwd>> = None;
-        let mut signatures = Vec::new();
-        for item in &mut *r {
-            let item = item?;
-            if item.kwd() != DIRECTORY_SIGNATURE {
-                return Err(EK::UnexpectedToken
-                    .with_msg(item.kwd().to_str())
-                    .at_pos(item.pos()));
-            }
-
-            let sig = Signature::from_item(&item)?;
-            if first_sig.is_none() {
-                first_sig = Some(item);
-            }
-            signatures.push(sig);
-        }
-
-        let end_pos = match first_sig {
-            None => return Err(EK::MissingToken.with_msg("directory-signature")),
-            // Unwrap should be safe because `first_sig` was parsed from `r`
-            #[allow(clippy::unwrap_used)]
-            Some(sig) => sig.offset_in(r.str()).unwrap() + "directory-signature ".len(),
-        };
-
-        // Find the appropriate digest.
-        let signed_str = &r.str()[start_pos..end_pos];
-        let remainder = &r.str()[end_pos..];
-        let (sha256, sha1) = match RS::flavor() {
-            ConsensusFlavor::Ns => (
-                None,
-                Some(ll::d::Sha1::digest(signed_str.as_bytes()).into()),
-            ),
-            ConsensusFlavor::Microdesc => (
-                Some(ll::d::Sha256::digest(signed_str.as_bytes()).into()),
-                None,
-            ),
-        };
-        let siggroup = SignatureGroup {
-            sha256,
-            sha1,
-            signatures,
-        };
-
-        let unval = UnvalidatedConsensus {
-            consensus,
-            siggroup,
-            n_authorities: None,
-        };
-        let lifetime = unval.consensus.header.hdr.lifetime.clone();
-        let delay = unval.consensus.header.hdr.voting_delay.unwrap_or((0, 0));
-        let dist_interval = time::Duration::from_secs(delay.1.into());
-        let starting_time = lifetime.valid_after - dist_interval;
-        let timebound = TimerangeBound::new(unval, starting_time..lifetime.valid_until);
-        Ok((signed_str, remainder, timebound))
-    }
-}
-
-/// A Microdesc consensus whose signatures have not yet been checked.
-///
-/// To validate this object, call set_n_authorities() on it, then call
-/// check_signature() on that result with the set of certs that you
-/// have.  Make sure only to provide authority certificates representing
-/// real authorities!
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
-#[derive(Debug, Clone)]
-pub struct UnvalidatedConsensus<RS> {
-    /// The consensus object. We don't want to expose this until it's
-    /// validated.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    consensus: Consensus<RS>,
-    /// The signatures that need to be validated before we can call
-    /// this consensus valid.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    siggroup: SignatureGroup,
-    /// The total number of authorities that we believe in.  We need
-    /// this information in order to validate the signatures, since it
-    /// determines how many signatures we need to find valid in `siggroup`.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    n_authorities: Option<u16>,
-}
-
-impl<RS> UnvalidatedConsensus<RS> {
-    /// Tell the unvalidated consensus how many authorities we believe in.
-    ///
-    /// Without knowing this number, we can't validate the signature.
-    #[must_use]
-    pub fn set_n_authorities(self, n_authorities: u16) -> Self {
-        UnvalidatedConsensus {
-            n_authorities: Some(n_authorities),
-            ..self
-        }
-    }
-
-    /// Return an iterator of all the certificate IDs that we might use
-    /// to validate this consensus.
-    pub fn signing_cert_ids(&self) -> impl Iterator<Item = AuthCertKeyIds> {
-        match self.key_is_correct(&[]) {
-            Ok(()) => Vec::new(),
-            Err(missing) => missing,
-        }
-        .into_iter()
-    }
-
-    /// Return the lifetime of this unvalidated consensus
-    pub fn peek_lifetime(&self) -> &Lifetime {
-        self.consensus.lifetime()
-    }
-
-    /// Return true if a client who believes in exactly the provided
-    /// set of authority IDs might might consider this consensus to be
-    /// well-signed.
-    ///
-    /// (This is the case if the consensus claims to be signed by more than
-    /// half of the authorities in the list.)
-    pub fn authorities_are_correct(&self, authorities: &[&RsaIdentity]) -> bool {
-        self.siggroup.could_validate(authorities)
-    }
-
-    /// Return the number of relays in this unvalidated consensus.
-    ///
-    /// This function is unstable. It is only enabled if the crate was
-    /// built with the `experimental-api` feature.
-    #[cfg(feature = "experimental-api")]
-    pub fn n_relays(&self) -> usize {
-        self.consensus.relays.len()
-    }
-
-    /// Modify the list of relays in this unvalidated consensus.
-    ///
-    /// A use case for this is long-lasting custom directories. To ensure Arti can still quickly
-    /// build circuits when the directory gets old, a tiny churn file can be regularly obtained,
-    /// listing no longer available Tor nodes, which can then be removed from the consensus.
-    ///
-    /// This function is unstable. It is only enabled if the crate was
-    /// built with the `experimental-api` feature.
-    #[cfg(feature = "experimental-api")]
-    pub fn modify_relays<F>(&mut self, func: F)
-    where
-        F: FnOnce(&mut Vec<RS>),
-    {
-        func(&mut self.consensus.relays);
-    }
-}
-
-impl<RS> ExternallySigned<Consensus<RS>> for UnvalidatedConsensus<RS> {
-    type Key = [AuthCert];
-    type KeyHint = Vec<AuthCertKeyIds>;
-    type Error = Error;
-
-    fn key_is_correct(&self, k: &Self::Key) -> result::Result<(), Self::KeyHint> {
-        let (n_ok, missing) = self.siggroup.list_missing(k);
-        match self.n_authorities {
-            Some(n) if n_ok > (n / 2) as usize => Ok(()),
-            _ => Err(missing.iter().map(|cert| cert.key_ids).collect()),
-        }
-    }
-    fn is_well_signed(&self, k: &Self::Key) -> result::Result<(), Self::Error> {
-        match self.n_authorities {
-            None => Err(Error::from(internal!(
-                "Didn't set authorities on consensus"
-            ))),
-            Some(authority) => {
-                if self.siggroup.validate(authority, k) {
-                    Ok(())
-                } else {
-                    Err(EK::BadSignature.err())
-                }
-            }
-        }
-    }
-    fn dangerously_assume_wellsigned(self) -> Consensus<RS> {
-        self.consensus
-    }
-}
-
 impl SignatureGroup {
     // TODO: these functions are pretty similar and could probably stand to be
     // refactored a lot.
@@ -1912,10 +1302,10 @@ mod test {
     const CERTS: &str = include_str!("../../testdata/authcerts2.txt");
     const CONSENSUS: &str = include_str!("../../testdata/mdconsensus1.txt");
 
-    #[cfg(feature = "ns_consensus")]
-    const NS_CERTS: &str = include_str!("../../testdata/authcerts3.txt");
-    #[cfg(feature = "ns_consensus")]
-    const NS_CONSENSUS: &str = include_str!("../../testdata/nsconsensus1.txt");
+    #[cfg(feature = "plain-consensus")]
+    const PLAIN_CERTS: &str = include_str!("../../testdata2/cached-certs");
+    #[cfg(feature = "plain-consensus")]
+    const PLAIN_CONSENSUS: &str = include_str!("../../testdata2/cached-consensus");
 
     fn read_bad(fname: &str) -> String {
         use std::fs;
@@ -1995,18 +1385,18 @@ mod test {
     }
 
     #[test]
-    #[cfg(feature = "ns_consensus")]
+    #[cfg(feature = "plain-consensus")]
     fn parse_and_validate_ns() -> Result<()> {
         use tor_checkable::{SelfSigned, Timebound};
         let mut certs = Vec::new();
-        for cert in AuthCert::parse_multiple(NS_CERTS)? {
+        for cert in AuthCert::parse_multiple(PLAIN_CERTS)? {
             let cert = cert?.check_signature()?.dangerously_assume_timely();
             certs.push(cert);
         }
         let auth_ids: Vec<_> = certs.iter().map(|c| &c.key_ids().id_fingerprint).collect();
-        assert_eq!(certs.len(), 3);
+        assert_eq!(certs.len(), 4);
 
-        let (_, _, consensus) = NsConsensus::parse(NS_CONSENSUS)?;
+        let (_, _, consensus) = PlainConsensus::parse(PLAIN_CONSENSUS)?;
         let consensus = consensus.dangerously_assume_timely().set_n_authorities(3);
         // The set of authorities we know _could_ validate this cert.
         assert!(consensus.authorities_are_correct(&auth_ids));
