@@ -51,7 +51,7 @@
 mod changes;
 mod graph;
 
-use anyhow::{anyhow, Context, Result};
+use anyhow::{Context, Result, anyhow};
 use std::collections::{HashMap, HashSet};
 use std::path::{Path, PathBuf};
 use toml_edit::{DocumentMut, Item, Table, Value};
@@ -216,7 +216,7 @@ impl Crate {
             };
 
             if !graph.contains_edge("full", wanted.as_str()) {
-                w(format!("full should contain {}. Fixing.", wanted));
+                w(format!("full should contain {wanted}. Fixing."));
                 changes.push(Change::AddExternalEdge("full".to_string(), wanted));
             }
         }
@@ -236,7 +236,9 @@ impl Crate {
             }
             // Every feature that depends on `__is_experimental` is reachable from `experimental`.
             for f in is_experimental.difference(&reachable_from_experimental) {
-                w(format!("{f} is marked as __is_experimental, but is not reachable from experimental. Fixing."));
+                w(format!(
+                    "{f} is marked as __is_experimental, but is not reachable from experimental. Fixing."
+                ));
                 changes.push(Change::AddEdge("experimental".into(), f.clone()))
             }
 
@@ -380,7 +382,10 @@ impl Crate {
 }
 
 /// Look at a toplevel Cargo.toml and find all of the paths in workplace.members
-fn list_crate_paths(toplevel: impl AsRef<Path>) -> Result<Vec<String>> {
+fn list_crate_paths(
+    toplevel: impl AsRef<Path>,
+    exclusion_prefixes: &[String],
+) -> Result<Vec<String>> {
     let s = std::fs::read_to_string(toplevel.as_ref())?;
     let toml_doc = s.parse::<DocumentMut>()?;
     Ok(toml_doc["workspace"]["members"]
@@ -392,21 +397,28 @@ fn list_crate_paths(toplevel: impl AsRef<Path>) -> Result<Vec<String>> {
                 .expect("Some member of workplace.members is not a string!?")
                 .to_owned()
         })
+        .filter(|s| {
+            // Iterate through all exclusion prefixes and check if there isn't one that `s` starts with.
+            !exclusion_prefixes
+                .iter()
+                .any(|prefix| s.starts_with(prefix))
+        })
         .collect())
 }
 
 fn main() -> Result<()> {
     let mut pargs = pico_args::Arguments::from_env();
-    const HELP: &str = "fixup-features [--no-annotate] <toplevel Cargo.toml>";
+    const HELP: &str = "fixup-features [--no-annotate] [--exclude <PREFIX1> --exclude <PREFIX2> ...] <toplevel Cargo.toml>";
 
     if pargs.contains(["-h", "--help"]) {
-        println!("{}", HELP);
+        println!("{HELP}");
         return Ok(());
     }
     let no_annotate = pargs.contains("--no-annotate");
+    let exclusion_prefixes: Vec<String> = pargs.values_from_str("--exclude").unwrap();
     let toplevel_toml_file: PathBuf = pargs.free_from_str()?;
     if !pargs.finish().is_empty() {
-        println!("{}", HELP);
+        println!("{HELP}");
         return Ok(());
     }
 
@@ -416,7 +428,7 @@ fn main() -> Result<()> {
         .to_path_buf();
     let mut crates = Vec::new();
     let mut crate_info = HashMap::new();
-    for p in list_crate_paths(&toplevel_toml_file)? {
+    for p in list_crate_paths(&toplevel_toml_file, &exclusion_prefixes)? {
         let mut crate_toml_path = toplevel_dir.clone();
         crate_toml_path.push(p);
         crate_toml_path.push("Cargo.toml");
